library value_broadcast;

class ValueBroadcast<T>{
  ///A Set is selected over ObserverList due to optimization towards low listener count.
  final Set<Function(T)> _listeners = Set<Function(T)>();

  ///High level functions.
  void addListener(final Function(T) listener) => _listeners.add(listener);
  void removeListener(final Function(T) listener) => _listeners.remove(listener);

  ValueBroadcast(this._value);

  ///Private due to [value] setter override, otherwise throws as already defined.
  T _value;

  T get value => _value;

  set value(final T update){
    if(_value != update){
      _value = update;
      _listeners.forEach((final Function(T) listener) => listener(update));
    }
  }
}